/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.iron;

import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2680;
import net.minecraft.class_3866;
import net.minecraft.class_3956;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.blockentity.MachineBaseBlockEntity;
import reborncore.common.util.IInventoryAccess;
import reborncore.common.util.RebornInventory;
import reborncore.common.util.ItemUtils;
import techreborn.events.TRRecipeHandler;
import techreborn.init.TRBlockEntities;

public class IronFurnaceBlockEntity extends MachineBaseBlockEntity
		implements InventoryProvider, IContainerProvider {

	public int tickTime;
	public RebornInventory<IronFurnaceBlockEntity> inventory = new RebornInventory<>(3, "IronFurnaceBlockEntity", 64, this, getInvetoryAccess()).withConfiguredAccess();
	public int fuel;
	public int fuelGague;
	public int progress;
	public int fuelScale = 160;
	int input1 = 0;
	int output = 1;
	int fuelslot = 2;
	boolean active = false;

	public IronFurnaceBlockEntity() {
		super(TRBlockEntities.IRON_FURNACE);
	}

	public int gaugeProgressScaled(final int scale) {
		return this.progress * scale / this.fuelScale;
	}

	public int gaugeFuelScaled(final int scale) {
		if (this.fuelGague == 0) {
			this.fuelGague = this.fuel;
			if (this.fuelGague == 0) {
				this.fuelGague = this.fuelScale;
			}
		}
		return this.fuel * scale / this.fuelGague;
	}

	@Override
	public void method_16896() {
		super.method_16896();
		if(field_11863.field_9236){
			return;
		}
		final boolean burning = this.isBurning();
		boolean updateInventory = false;
		if (this.fuel > 0) {
			this.fuel--;
			this.updateState();
		}
		if (this.fuel <= 0 && this.canSmelt()) {
			this.fuel = this.fuelGague = (int) (class_3866.method_11196().getOrDefault(inventory.method_5438(this.fuelslot).method_7909(), 0) * 1.25);
			if (this.fuel > 0) {
				// Fuel slot
				class_1799 fuelStack = inventory.method_5438(this.fuelslot);
				if (fuelStack.method_7909().method_7857()) {
					inventory.method_5447(this.fuelslot, new class_1799(fuelStack.method_7909().method_7858()));
				} else if (fuelStack.method_7947() > 1) {
					inventory.shrinkSlot(this.fuelslot, 1);
				} else if (fuelStack.method_7947() == 1) {
					inventory.method_5447(this.fuelslot, class_1799.field_8037);
				}
				updateInventory = true;
			}
		}
		if (this.isBurning() && this.canSmelt()) {
			this.progress++;
			if (this.progress >= this.fuelScale) {
				this.progress = 0;
				this.cookItems();
				updateInventory = true;
			}
		} else {
			this.progress = 0;
		}
		if (burning != this.isBurning()) {
			updateInventory = true;
		}
		if (updateInventory) {
			this.method_5431();
		}
	}

	public void cookItems() {
		if (this.canSmelt()) {
			final class_1799 itemstack = TRRecipeHandler.getMatchingRecipes(field_11863, class_3956.field_17546, inventory.method_5438(this.input1));

			if (inventory.method_5438(this.output).method_7960()) {
				inventory.method_5447(this.output, itemstack.method_7972());
			} else if (inventory.method_5438(this.output).method_7962(itemstack)) {
				inventory.method_5438(this.output).method_7933(itemstack.method_7947());
			}
			if (inventory.method_5438(this.input1).method_7947() > 1) {
				inventory.shrinkSlot(this.input1, 1);
			} else {
				inventory.method_5447(this.input1, class_1799.field_8037);
			}
		}
	}

	public boolean canSmelt() {
		if (inventory.method_5438(this.input1).method_7960())
			return false;
		final class_1799 itemstack = TRRecipeHandler.getMatchingRecipes(field_11863, class_3956.field_17546, inventory.method_5438(this.input1));
		if (itemstack.method_7960())
			return false;
		if (inventory.method_5438(this.output).method_7960())
			return true;
		if (!inventory.method_5438(this.output).method_7962(itemstack))
			return false;
		final int result = inventory.method_5438(this.output).method_7947() + itemstack.method_7947();
		return result <= inventory.getStackLimit() && result <= itemstack.method_7914();
	}

	public boolean isBurning() {
		return this.fuel > 0;
	}

	public class_1799 getResultFor(final class_1799 stack) {
		final class_1799 result = TRRecipeHandler.getMatchingRecipes(field_11863, class_3956.field_17546, stack);
		if (!result.method_7960()) {
			return result.method_7972();
		}
		return class_1799.field_8037;
	}

	public void updateState() {
		final class_2680 BlockStateContainer = this.field_11863.method_8320(this.field_11867);
		if (BlockStateContainer.method_11614() instanceof BlockMachineBase) {
			final BlockMachineBase blockMachineBase = (BlockMachineBase) BlockStateContainer.method_11614();
			if (BlockStateContainer.method_11654(BlockMachineBase.ACTIVE) != this.fuel > 0)
				blockMachineBase.setActive(this.fuel > 0, this.field_11863, this.field_11867);
		}
	}

	@Override
	public RebornInventory<IronFurnaceBlockEntity> getInventory() {
		return this.inventory;
	}

	public static IInventoryAccess<IronFurnaceBlockEntity> getInvetoryAccess(){
		return (slotID, stack, face, direction, blockEntity) -> {
			if(direction == IInventoryAccess.AccessDirection.INSERT){
				boolean isFuel = class_3866.method_11195(stack);
				if(isFuel){
					class_1799 fuelSlotStack = blockEntity.inventory.method_5438(blockEntity.fuelslot);
					if(fuelSlotStack.method_7960() || ItemUtils.isItemEqual(stack, fuelSlotStack, true, true) && fuelSlotStack.method_7914() != fuelSlotStack.method_7947()){
						return slotID == blockEntity.fuelslot;
					}
				}
				return slotID != blockEntity.output;
			}
			return true;
		};
	}
	
	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	public int getBurnTime() {
		return this.fuel;
	}

	public void setBurnTime(final int burnTime) {
		this.fuel = burnTime;
	}

	public int getTotalBurnTime() {
		return this.fuelGague;
	}

	public void setTotalBurnTime(final int totalBurnTime) {
		this.fuelGague = totalBurnTime;
	}

	public int getProgress() {
		return progress;
	}

	public void setProgress(int progress) {
		this.progress = progress;
	}

	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("ironfurnace").player(player.field_7514).inventory(8, 84).hotbar(8, 142)
				.addInventory().blockEntity(this).fuelSlot(2, 56, 53).slot(0, 56, 17).outputSlot(1, 116, 35)
				.syncIntegerValue(this::getBurnTime, this::setBurnTime)
				.syncIntegerValue(this::getProgress, this::setProgress)
				.syncIntegerValue(this::getTotalBurnTime, this::setTotalBurnTime).addInventory().create(this, syncID);
	}
}
