/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.multiblock;

import net.minecraft.class_1657;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2586;
import net.minecraft.class_3614;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.multiblock.IMultiblockPart;
import reborncore.common.recipes.RecipeCrafter;
import reborncore.common.registration.RebornRegister;
import reborncore.common.registration.config.ConfigRegistry;
import reborncore.common.util.RebornInventory;
import techreborn.TechReborn;
import techreborn.blocks.BlockMachineCasing;
import techreborn.init.ModRecipes;
import techreborn.init.TRContent;
import techreborn.init.TRBlockEntities;
import techreborn.multiblocks.MultiBlockCasing;
import techreborn.blockentity.GenericMachineBlockEntity;
import techreborn.blockentity.MachineCasingBlockEntity;

@RebornRegister(TechReborn.MOD_ID)
public class IndustrialBlastFurnaceBlockEntity extends GenericMachineBlockEntity implements IContainerProvider {
	
	@ConfigRegistry(config = "machines", category = "industrial_furnace", key = "IndustrialFurnaceMaxInput", comment = "Industrial Blast Furnace Max Input (Value in EU)")
	public static int maxInput = 128;
	@ConfigRegistry(config = "machines", category = "industrial_furnace", key = "IndustrialFurnaceMaxEnergy", comment = "Industrial Blast Furnace Max Energy (Value in EU)")
	public static int maxEnergy = 40_000;

	public MultiblockChecker multiblockChecker;
	private int cachedHeat;

	public IndustrialBlastFurnaceBlockEntity() {
		super(TRBlockEntities.INDUSTRIAL_BLAST_FURNACE, "IndustrialBlastFurnace", maxInput, maxEnergy, TRContent.Machine.INDUSTRIAL_BLAST_FURNACE.block, 4);
		final int[] inputs = new int[] { 0, 1 };
		final int[] outputs = new int[] { 2, 3 };
		this.inventory = new RebornInventory<>(5, "IndustrialBlastFurnaceBlockEntity", 64, this).withConfiguredAccess();
		this.crafter = new RecipeCrafter(ModRecipes.BLAST_FURNACE, this, 2, 2, this.inventory, inputs, outputs);
	}
	
	public int getHeat() {
		if (!getMutliBlock()){
			return 0;
		}
		
		// Bottom center of multiblock
		final class_2338 location = field_11867.method_10079(getFacing().method_10153(), 2);
		final class_2586 blockEntity = field_11863.method_8321(location);

		if (blockEntity instanceof MachineCasingBlockEntity) {
			if (((MachineCasingBlockEntity) blockEntity).isConnected()
					&& ((MachineCasingBlockEntity) blockEntity).getMultiblockController().isAssembled()) {
				final MultiBlockCasing casing = ((MachineCasingBlockEntity) blockEntity).getMultiblockController();

				int heat = 0;

				// Bottom center shouldn't have any blockEntity entities below it
				if (field_11863.method_8320(new class_2338(location.method_10263(), location.method_10264() - 1, location.method_10260()))
						.method_11614() == blockEntity.method_10997().method_8320(blockEntity.method_11016()).method_11614()) {
					return 0;
				}

				for (final IMultiblockPart part : casing.connectedParts) {
					heat += BlockMachineCasing.getHeatFromState(part.method_11010());
				}

				if (field_11863.method_8320(location.method_10079(class_2350.field_11036, 1)).method_11614().method_9539().equals("blockEntity.lava")
						&& field_11863.method_8320(location.method_10079(class_2350.field_11036, 2)).method_11614().method_9539().equals("blockEntity.lava")) {
					heat += 500;
				}
				return heat;
			}
		}

		return 0;
	}
	
	public boolean getMutliBlock() {
		if(multiblockChecker == null){
			return false;
		}
		final boolean layer0 = multiblockChecker.checkRectY(1, 1, MultiblockChecker.CASING_ANY, MultiblockChecker.ZERO_OFFSET);
		final boolean layer1 = multiblockChecker.checkRingY(1, 1, MultiblockChecker.CASING_ANY, new class_2338(0, 1, 0));
		final boolean layer2 = multiblockChecker.checkRingY(1, 1, MultiblockChecker.CASING_ANY, new class_2338(0, 2, 0));
		final boolean layer3 = multiblockChecker.checkRectY(1, 1, MultiblockChecker.CASING_ANY, new class_2338(0, 3, 0));
		final class_3614 centerBlock1 = multiblockChecker.getBlock(0, 1, 0).method_11620();
		final class_3614 centerBlock2 = multiblockChecker.getBlock(0, 2, 0).method_11620();
		final boolean center1 = (centerBlock1 == class_3614.field_15959 || centerBlock1 == class_3614.field_15922);
		final boolean center2 = (centerBlock2 == class_3614.field_15959 || centerBlock2 == class_3614.field_15922);
		return layer0 && layer1 && layer2 && layer3 && center1 && center2;
	}
	
	public void setHeat(final int heat) {
		cachedHeat = heat;
	}

	public int getCachedHeat() {
		return cachedHeat;
	}
	
	// TileGenericMachine
	@Override
	public void method_16896() {
		if (multiblockChecker == null) {
			final class_2338 downCenter = field_11867.method_10079(getFacing().method_10153(), 2);
			multiblockChecker = new MultiblockChecker(field_11863, downCenter);
		}
		
		if (getMutliBlock()){
			super.method_16896();
		}		
	}
	
	// IContainerProvider
	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("blastfurnace").player(player.field_7514).inventory().hotbar().addInventory()
				.blockEntity(this).slot(0, 50, 27).slot(1, 50, 47).outputSlot(2, 93, 37).outputSlot(3, 113, 37)
				.energySlot(4, 8, 72).syncEnergyValue().syncCrafterValue()
				.syncIntegerValue(this::getHeat, this::setHeat).addInventory().create(this, syncID);
	}

}
