/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items;

import io.github.prospector.silk.fluid.FluidInstance;
import net.minecraft.class_1297;
import net.minecraft.class_1761;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2371;
import net.minecraft.class_2487;
import net.minecraft.class_3611;
import org.apache.commons.lang3.Validate;
import reborncore.common.fluid.container.GenericFluidContainer;
import reborncore.common.util.ItemNBTHelper;
import techreborn.TechReborn;
import techreborn.init.TRContent;
import techreborn.utils.FluidUtils;

/**
 * Created by modmuss50 on 17/05/2016.
 */
public class ItemDynamicCell extends class_1792 implements GenericFluidContainer<class_1799> {

	public static final int CAPACITY = 1000;

	public ItemDynamicCell() {
		super(new class_1792.class_1793().method_7889(1).method_7892(TechReborn.ITEMGROUP));
	}

	@Override
	public void method_7888(class_1799 stack, class_1937 worldIn, class_1297 entityIn, int itemSlot, boolean isSelected) {
		//Clearing tag because ItemUtils.isItemEqual doesn't handle tags ForgeCaps and display
		//And breaks ability to use in recipes
		//TODO: Property ItemUtils.isItemEquals tags equality handling?
		if (stack.method_7985()) {
			class_2487 tag = stack.method_7969();
			if (tag.method_10546() != 1 || tag.method_10545("Fluid")) {
				class_2487 clearTag = new class_2487();
				clearTag.method_10566("Fluid", tag.method_10562("Fluid"));
				stack.method_7980(clearTag);
			}
		}
	}

	@Override
	public void method_7850(class_1761 tab, class_2371<class_1799> subItems) {
		if (!method_7877(tab)) {
			return;
		}
		subItems.add(getEmptyCell(1));
		for (class_3611 fluid : FluidUtils.getAllFluids()) {
			subItems.add(getCellWithFluid(fluid));
		}
	}

//	@Override
//	public String getTranslationKey(ItemStack stack) {
//		FluidStack fluidStack = getFluidHandler(stack).getFluidInstance();
//		if (fluidStack == null)
//			return super.getTranslationKey(stack);
//		return StringUtils.t("item.techreborn.cell.fluid.name").replaceAll("\\$fluid\\$", fluidStack.getLocalizedName());
//	}

	public static class_1799 getCellWithFluid(class_3611 fluid, int stackSize) {
		Validate.notNull(fluid);
		class_1799 stack = new class_1799(TRContent.CELL);
		GenericFluidContainer<class_1799> fluidContainer = GenericFluidContainer.fromStack(stack);
		Validate.notNull(fluidContainer);
		fluidContainer.setFluid(stack, new FluidInstance(fluid, fluidContainer.getCapacity(stack)));
		stack.method_7939(stackSize);
		return stack;
	}

	public static class_1799 getEmptyCell(int amount) {
		return new class_1799(TRContent.CELL, amount);
	}

	public static class_1799 getCellWithFluid(class_3611 fluid) {
		return getCellWithFluid(fluid, 1);
	}


	@Override
	public void setFluid(class_1799 type, FluidInstance instance) {
		Validate.notNull(type, "ItemStack cannot be null!");
		class_2487 compoundTag = new class_2487();
		instance.toTag(compoundTag);
		ItemNBTHelper.getNBT(type).method_10566("fluid", compoundTag);
	}

	@Override
	public FluidInstance getFluidInstance(class_1799 type) {
		Validate.notNull(type, "ItemStack cannot be null!");
		if(ItemNBTHelper.getNBT(type).method_10545("fluid")){
			class_2487 compoundTag = ItemNBTHelper.getNBT(type).method_10562("fluid");
			return new FluidInstance(compoundTag);
		}
		return new FluidInstance();
	}

	@Override
	public int getCapacity(class_1799 type) {
		Validate.notNull(type, "ItemStack cannot be null!");
		return CAPACITY;
	}
}
