/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items.tool.advanced;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1761;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1810;
import net.minecraft.class_1834;
import net.minecraft.class_1890;
import net.minecraft.class_1893;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2371;
import net.minecraft.class_2680;
import net.minecraft.item.*;
import reborncore.api.power.IEnergyItemInfo;
import reborncore.common.powerSystem.ExternalPowerSystems;
import reborncore.common.powerSystem.ItemPowerManager;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.ItemDurabilityExtensions;
import reborncore.common.util.ItemUtils;
import techreborn.TechReborn;
import techreborn.config.ConfigTechReborn;
import techreborn.init.TRContent;

import java.util.Random;

public class ItemRockCutter extends class_1810 implements IEnergyItemInfo, ItemDurabilityExtensions {

	public static final int maxCharge = ConfigTechReborn.RockCutterCharge;
	public int transferLimit = 1_000;
	public int cost = 500;

	// 400k FE with 1k FE\t charge rate
	public ItemRockCutter() {
		super(class_1834.field_8930, 1, 1, new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP).method_7889(1));
	}

	// ItemPickaxe
	@Override
	public boolean method_7856(class_2680 state) {
		if (class_1802.field_8377.method_7856(state)) {
			return true;
		}
		return false;
	}

	@Override
	public float method_7865(class_1799 stack, class_2680 state) {
		if (new ItemPowerManager(stack).getEnergyStored() < cost) {
			return 2F;
		} else {
			return class_1802.field_8377.method_7865(stack, state);
		}
	}

	// ItemTool
	@Override
	public boolean method_7879(class_1799 stack, class_1937 worldIn, class_2680 blockIn, class_2338 pos, class_1309 entityLiving) {
		Random rand = new Random();
		if (rand.nextInt(class_1890.method_8225(class_1893.field_9119, stack) + 1) == 0) {
			ItemPowerManager capEnergy = new ItemPowerManager(stack);

			capEnergy.extractEnergy(cost, false);
			ExternalPowerSystems.requestEnergyFromArmor(capEnergy, entityLiving);
		}
		return true;
	}

//	@Override
//	public int getHarvestLevel(ItemStack stack, ToolType toolType, @Nullable PlayerEntity player, @Nullable BlockState blockState) {
//		if (!stack.hasEnchantments()) {
//			stack.addEnchantment(Enchantments.SILK_TOUCH, 1);
//		}
//		return super.getHarvestLevel(stack, toolType, player, blockState);
//	}

	// Item
	@Override
	public boolean method_7878(class_1799 stack, class_1799 stack2) {
		return false;
	}

	@Override
	public void method_7843(class_1799 stack, class_1937 worldIn, class_1657 playerIn) {
		if (!stack.method_7942()) {
			stack.method_7978(class_1893.field_9099, 1);
		}
	}

	@Override
	public double getDurabilityForDisplay(class_1799 stack) {
		return 1 - ItemUtils.getPowerForDurabilityBar(stack);
	}

	@Override
	public boolean showDurabilityBar(class_1799 stack) {
		return true;
	}

	@Override
	public int getRGBDurabilityForDisplay(class_1799 stack) {
		return PowerSystem.getDisplayPower().colour;
	}

	@Environment(EnvType.CLIENT)
	@Override
	public void method_7850(class_1761 par2ItemGroup, class_2371<class_1799> itemList) {
		if (!method_7877(par2ItemGroup)) {
			return;
		}
		class_1799 uncharged = new class_1799(this);
		uncharged.method_7978(class_1893.field_9099, 1);
		class_1799 charged = new class_1799(TRContent.ROCK_CUTTER);
		charged.method_7978(class_1893.field_9099, 1);
		ItemPowerManager capEnergy = new ItemPowerManager(charged);
		capEnergy.setEnergyStored(capEnergy.getMaxEnergyStored());

		itemList.add(uncharged);
//		itemList.add(charged);
	}

	// IEnergyItemInfo
	@Override
	public int getCapacity() {
		return maxCharge;
	}

	@Override
	public int getMaxInput() {
		return transferLimit;
	}

	@Override
	public int getMaxOutput() {
		return 0;
	}
}
